package ex10;

import java.util.ArrayList;
import java.util.List;

public class PersonUtil {

    private static final int LEGAL_AGE = 18;
    private static final String FEMALE = "W";

    public Person getOldest(List<Person> persons) {
        ensureNotNull(persons);
        ensureNotEmpty(persons);

        Person oldestPerson = persons.get(0);
        for (Person person : persons) {
            if (person.getAge() > oldestPerson.getAge()) {
                oldestPerson = person;
            }
        }

        return oldestPerson;
    }

    public List<Person> getPersonsInLegalAge(List<Person> persons) {
        ensureNotNull(persons);

        List<Person> adults = new ArrayList<Person>();
        for (Person person : persons) {
            if (person.getAge() >= LEGAL_AGE) {
                adults.add(person);
            }
        }

        return adults;
    }

    public List<Person> getWomen(List<Person> persons) {
        ensureNotNull(persons);

        List<Person> women = new ArrayList<Person>();
        for (Person person : persons) {
            if (FEMALE.equals(person.getGender())) {
                women.add(person);
            }
        }

        return women;
    }

    public List<Person> getPersonsWhoLiveIn(String city, List<Person> persons) {
        ensureNotNull(city);
        ensureNotNull(persons);

        List<Person> personsFromCity = new ArrayList<Person>();
        for (Person person : persons) {
            if (person.getAddress().getTown().equals(city)) {
                personsFromCity.add(person);
            }
        }

        return personsFromCity;
    }

    private void ensureNotEmpty(List<Person> persons) {
        if (persons.isEmpty()) throw new IllegalArgumentException();
    }

    private void ensureNotNull(Object object) {
        if (object == null) throw new IllegalArgumentException();
    }
}
