package ex11;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

public class InoviceRowGenerator {

    private InvoiceRowDao dao;

    public void generateRowsFor(BigDecimal amount, Date periodStart, Date periodEnd) {

        if (periodEnd.before(periodStart)) throw new IllegalArgumentException();

        List<Payment> payments = paymentsForDates(periodStart, periodEnd);

        for (Payment payment : payments) {
            payment.amount = onePaymentAmount(amount, payments);
        }

        if (doesNotDevideEqually(amount, paymentCount(payments))) {
            increaseLastPaymentByOne(payments);
        }

        for (Payment payment : payments) {
            dao.save(new InvoiceRow(payment.amount, payment.date));
        }
    }

    private BigDecimal onePaymentAmount(BigDecimal amount,
            List<Payment> payments) {
        return amount.divide(
                paymentCount(payments), RoundingMode.FLOOR);
    }

    private BigDecimal paymentCount(List<Payment> payments) {
        return new BigDecimal(payments.size());
    }

    private void increaseLastPaymentByOne(List<Payment> payments) {
        Payment lastPayment = payments.get(payments.size() - 1);
        lastPayment.amount = lastPayment.amount.add(BigDecimal.ONE);
    }

    private boolean doesNotDevideEqually(BigDecimal amount, BigDecimal divisor) {
        return amount.remainder(divisor).compareTo(BigDecimal.ZERO) != 0;
    }

    private List<Payment> paymentsForDates(Date periodStart, Date periodEnd) {
        List<Payment> payments = new ArrayList<>();

        Date runner = periodStart;
        do {
            payments.add(new Payment(runner));
            runner = getBeginningOfNextMonth(runner);

        } while (!runner.after(periodEnd));
        return payments;
    }

    private Date getBeginningOfNextMonth(Date date) {
        Calendar c = GregorianCalendar.getInstance();
        c.setTime(date);

        do {
            c.add(Calendar.DAY_OF_MONTH, 1);
        } while (c.get(Calendar.DAY_OF_MONTH) != 1);

        return c.getTime();
    }

    private class Payment {
        public Payment(Date date) {
            this.date = date;
        }
        Date date;
        BigDecimal amount;
        @Override
        public String toString() {
            return MessageFormat.format("{0} - {1}", date, amount);
        }
    }
}
