package ex11;

import static org.hamcrest.CoreMatchers.*;
import static org.mockito.Matchers.argThat;
import static org.mockito.Mockito.*;
import static org.hamcrest.MatcherAssert.*;

import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.hamcrest.*;
import org.junit.Test;
import org.laughingpanda.beaninject.Inject;

public class InvoiceRowGeneratorTest {

    @Test
    public void amountsAreCorrect() throws Exception {

        InvoiceRowDao dao = mock(InvoiceRowDao.class);
        InoviceRowGenerator generator = new InoviceRowGenerator();
        Inject.bean(generator).with(dao);

        generator.generateRowsFor(new BigDecimal(10), asDate("2013-02-15"), asDate("2013-04-02"));

        verify(dao, times(2)).save(argThat(hasAmountOf(3)));
        verify(dao).save(argThat(hasAmountOf(4)));

        verifyNoMoreInteractions(dao);
    }

    @Test
    public void datesAreCorrect() throws Exception {
        // ...
    }

    @Test
    public void testMatcherForAmount() {
        assertThat(new InvoiceRow(BigDecimal.ONE, new Date()),
                hasAmountOf(1));

        assertThat(new InvoiceRow(BigDecimal.ONE, new Date()),
                not(hasAmountOf(0)));
    }

    private Matcher<InvoiceRow> hasAmountOf(int amount) {
        final BigDecimal storedAmount = new BigDecimal(amount);

        return new TypeSafeMatcher<InvoiceRow>() {

            @Override
            protected boolean matchesSafely(InvoiceRow item) {
                return storedAmount.compareTo(item.amount) == 0;
            }

            @Override
            public void describeTo(Description description) {
                description.appendText(storedAmount.toPlainString());
            }
        };
    }

    private static Date asDate(String date) {
        try {
            return new SimpleDateFormat("yyyy-MM-dd").parse(date);
        } catch (ParseException e) {
            throw new RuntimeException(e);
        }
    }

}